from os.path import expanduser


API_URL = "https://api.threatrecon.co/api/v1/search"
# TODO : validate response codes with formal API docs
API_RESPONSES = {
    -99:    "General API Response Error",
    -1:     "Invalid API Key",
}

API_FIELDS = [
    'indicator',
    'type',
    'reference',
    'source',
    'killchain',
    'firstseen',
    'lastseen',
    'attribution',
    'processtype',
    'rrname',
    'rdata',
    'country',
    'rootnode',
    'tags',
    'comment',
    'confidence',
    'id',
]

API_FILENAME = ".threatrecon-apikey"

class APIError(Exception):
    """
    User-defined exception for API problems.
    """
    def __init__(self, value):
        self.value = value

    def __str__(self):
        return "%s (%d)" % (
            API_RESPONSES.get(self.value, "Unknown API Error"), self.value
        )


def get_api_key(fn=API_FILENAME):
    """
    Will return the api key stored in API_FILENAME (default
    ~/.threatrecon-apikey) or False
    if any error was encountered.
    """
    try:
        apifile = "%s/%s" % (expanduser("~"), fn)
        f = open(apifile, "r")
        api_key = f.read()
        return api_key[:-1]
    except:
        return False



import urllib2
import httplib
import ssl
import socket


# from http://bugs.python.org/issue11220
class HTTPSConnectionV3(httplib.HTTPSConnection):
    def __init__(self, *args, **kwargs):
        httplib.HTTPSConnection.__init__(self, *args, **kwargs)

    def connect(self):
        sock = socket.create_connection((self.host, self.port), self.timeout)
        if self._tunnel_host:
            self.sock = sock
            self._tunnel()
        try:
            self.sock = ssl.wrap_socket(
                sock,
                self.key_file,
                self.cert_file,
                ssl_version=ssl.PROTOCOL_SSLv3
            )
        except ssl.SSLError:
            print("Trying SSLv3.")
            self.sock = ssl.wrap_socket(
                sock,
                self.key_file,
                self.cert_file,
                ssl_version=ssl.PROTOCOL_SSLv23
            )


class HTTPSHandlerV3(urllib2.HTTPSHandler):
    def https_open(self, req):
        return self.do_open(HTTPSConnectionV3, req)



import json
import urllib
import urllib2
from sslv3 import HTTPSHandlerV3
from datetime import datetime
from dateutil import parser
from api import API_FIELDS, APIError


def raw_query_threat_recon(indicator, api_key):
    """
    Uses an indicator and the api key to query the threat recon
    database. Returns a list of python dicts corresponding to
    the JSON output from the server (results only). If no results,
    returns an empty list.

    Will throw APIError exception if the ResponseCode < 0.
    """
    params = urllib.urlencode({'api_key': api_key, 'indicator': indicator})
    urllib2.install_opener(urllib2.build_opener(HTTPSHandlerV3()))

    f = urllib2.urlopen("https://api.threatrecon.co/api/v1/search", params)
    data = json.load(f)
    response = data.get("ResponseCode", -99)
    if response < 0:
        raise APIError(response)

    results = data.get("Results", [])
    return results or []


def query_threat_recon(indicator, api_key):
    """
    Calls _raw_query_threat_recon and returns a list of TRIndicator
    objects with appropriate attributes set.
    """
    results = []
    for r in raw_query_threat_recon(indicator, api_key):
        i = TRIndicator(**r)
        i._query_root = (
            i.rootnode == '' and
            i.indicator == indicator
        )

        results.append(i)
    return results


class TRIndicator(object):
    """
    Default class for Threat Recon Indicator objects.
    """
    # replaces Result object

    def __repr__(self):
        return "TRIndicator %s [id %d type %s confidence %d%s]" % (
            self.indicator,
            self.id,
            self.type,
            self.confidence,
            " ***QUERY ROOT***" if self._query_root else ""
        )

    @property
    def verbose(self):
        r = "\n"
        for i in API_FIELDS:
            r += "%s: %s\n" % (i, getattr(self, i, None))
        if self._query_root:
            r += "***This is the query root record***\n"

        return r

    def __init__(self, *args, **kwargs):
        kwargsl = {k.lower(): v for k, v in kwargs.items()}
        for i in API_FIELDS:
            if i in kwargsl:
                v = kwargsl[i]
                if (i == 'firstseen') or (i == 'lastseen'):
                    try:
                        v = parser.parse(v)
                    except TypeError:
                        v = datetime.min
            else:
                v = None
            setattr(self, i, v)



from setuptools import setup 

setup(
    name='threatrecon',
    version='1.0.0',
    author='Wapack Labs',
    author_email='chall@wapacklabs.com',
    packages=['threatrecon', ],
    description='Python API module for Wapack Labs ThreatRecon service.',
    install_requires=[
        "python-dateutil >= 2.2",
    ],
)
